c2py
c2py is a lightweight C++20 library that provides the runtime support for the Python bindings generated by clair-c2py.
TBW : Calling Python from C++.
c2py is a lightweight C++20 library that provides the runtime support for the Python bindings generated by clair-c2py.
TBW : Calling Python from C++.